@@ -1,5 +1,4 @@
 extern crate roxmltree;
-#[macro_use] extern crate pretty_assertions;
 
 use roxmltree::*;
 
@@ -87,7 +86,13 @@ fn get_pi() {
 
     let doc = Document::parse(data).unwrap();
     let node = doc.root().first_child().unwrap();
-    assert_eq!(node.pi(), Some(PI { target: "target", value: Some("value") }));
+    assert_eq!(
+        node.pi(),
+        Some(PI {
+            target: "target",
+            value: Some("value")
+        })
+    );
 }
 
 #[test]
@@ -115,11 +120,15 @@ fn lookup_namespace_uri() {
 
     let doc = Document::parse(data).unwrap();
     let node = doc.root_element();
-    assert_eq!(node.lookup_namespace_uri(Some("n1")), Some("http://www.w3.org"));
+    assert_eq!(
+        node.lookup_namespace_uri(Some("n1")),
+        Some("http://www.w3.org")
+    );
     assert_eq!(node.lookup_namespace_uri(None), Some("http://www.w4.org"));
     assert_eq!(node.lookup_namespace_uri(Some("n2")), None);
 }
 
+#[cfg(feature = "positions")]
 #[test]
 fn text_pos_01() {
     let data = "\
@@ -132,26 +141,26 @@ fn text_pos_01() {
     let doc = Document::parse(data).unwrap();
     let node = doc.root_element();
 
-    assert_eq!(doc.text_pos_at(node.range().start), TextPos::new(1, 1));
-    assert_eq!(doc.text_pos_at(node.range().end), TextPos::new(4, 5));
+    assert_eq!(doc.text_pos_at(doc.root().position()), TextPos::new(1, 1));
+
+    assert_eq!(doc.text_pos_at(node.position()), TextPos::new(1, 1));
 
     if let Some(attr) = node.attribute_node("a") {
-        assert_eq!(doc.text_pos_at(attr.range().start), TextPos::new(1, 4));
-        assert_eq!(doc.text_pos_at(attr.value_range().start), TextPos::new(1, 7));
+        assert_eq!(doc.text_pos_at(attr.position()), TextPos::new(1, 4));
     }
 
     // first child is a text/whitespace, not a comment
     let comm = node.first_child().unwrap().next_sibling().unwrap();
-    assert_eq!(doc.text_pos_at(comm.range().start), TextPos::new(2, 5));
+    assert_eq!(doc.text_pos_at(comm.position()), TextPos::new(2, 5));
 
     let p = comm.next_sibling().unwrap().next_sibling().unwrap();
-    assert_eq!(doc.text_pos_at(p.range().start), TextPos::new(3, 5));
-    assert_eq!(doc.text_pos_at(p.range().end), TextPos::new(3, 16));
+    assert_eq!(doc.text_pos_at(p.position()), TextPos::new(3, 5));
 
     let text = p.first_child().unwrap();
-    assert_eq!(doc.text_pos_at(text.range().start), TextPos::new(3, 8));
+    assert_eq!(doc.text_pos_at(text.position()), TextPos::new(3, 8));
 }
 
+#[cfg(feature = "positions")]
 #[test]
 fn text_pos_02() {
     let data = "<n1:e xmlns:n1='http://www.w3.org' n1:a='b'/>";
@@ -159,14 +168,14 @@ fn text_pos_02() {
     let doc = Document::parse(data).unwrap();
     let node = doc.root_element();
 
-    assert_eq!(doc.text_pos_at(node.range().start), TextPos::new(1, 1));
+    assert_eq!(doc.text_pos_at(node.position()), TextPos::new(1, 1));
 
     if let Some(attr) = node.attribute_node(("http://www.w3.org", "a")) {
-        assert_eq!(doc.text_pos_at(attr.range().start), TextPos::new(1, 36));
-        assert_eq!(doc.text_pos_at(attr.value_range().start), TextPos::new(1, 42));
+        assert_eq!(doc.text_pos_at(attr.position()), TextPos::new(1, 36));
     }
 }
 
+#[cfg(feature = "positions")]
 #[test]
 fn text_pos_03() {
     let data = "\
@@ -177,14 +186,67 @@ fn text_pos_03() {
     let doc = Document::parse(data).unwrap();
     let node = doc.root_element();
 
-    assert_eq!(doc.text_pos_at(node.range().start), TextPos::new(2, 1));
-    assert_eq!(doc.text_pos_at(node.range().end), TextPos::new(2, 5));
+    assert_eq!(doc.text_pos_at(node.position()), TextPos::new(2, 1));
+}
+
+#[test]
+fn next_sibling_element_01() {
+    let data = "<root><a/><b/><c/></root>";
+
+    let doc = roxmltree::Document::parse(data).unwrap();
+
+    let root = doc.root_element();
+    let a = root.first_element_child().unwrap();
+    let b = a.next_sibling_element().unwrap();
+    let c = b.next_sibling_element().unwrap();
+    assert!(c.next_sibling_element().is_none());
+
+    assert_eq!(root.tag_name().name(), "root");
+    assert_eq!(a.tag_name().name(), "a");
+    assert_eq!(b.tag_name().name(), "b");
+    assert_eq!(c.tag_name().name(), "c");
+}
+
+#[test]
+fn next_prev_element_01() {
+    let data = "<root><a/><b/><c/></root>";
+
+    let doc = roxmltree::Document::parse(data).unwrap();
+
+    let root = doc.root_element();
+    let c = root.last_element_child().unwrap();
+    let b = c.prev_sibling_element().unwrap();
+    let a = b.prev_sibling_element().unwrap();
+    assert!(a.prev_sibling_element().is_none());
+
+    assert_eq!(root.tag_name().name(), "root");
+    assert_eq!(a.tag_name().name(), "a");
+    assert_eq!(b.tag_name().name(), "b");
+    assert_eq!(c.tag_name().name(), "c");
+}
+
+#[test]
+fn nodes_document_order() {
+    let data = "<root><a/><b/><c/></root>";
+
+    let doc = roxmltree::Document::parse(data).unwrap();
+    let root = doc.root_element();
+    let a = root.first_element_child().unwrap();
+    let b = a.next_sibling_element().unwrap();
+    let c = b.next_sibling_element().unwrap();
+
+    let mut elems = vec![&b, &c, &a];
+    elems.sort();
+    assert!(elems[0] == &a);
+    assert!(elems[1] == &b);
+    assert!(elems[2] == &c);
 }
 
 #[test]
 fn lifetimes() {
     fn f<'a, 'd, F, R>(doc: &'a roxmltree::Document<'d>, fun: F) -> R
-        where F: Fn(&'a roxmltree::Document<'d>) -> R
+    where
+        F: Fn(&'a roxmltree::Document<'d>) -> R,
     {
         fun(doc)
     }
@@ -207,3 +269,15 @@ fn lifetimes() {
     let _ = f(&doc, |d| d.root().tail());
     let _ = f(&doc, |d| d.root().pi());
 }
+
+#[test]
+fn tag_name_lifetime() {
+    fn get_tag_name<'a, 'input>(node: &'a Node<'a, 'input>) -> &'input str {
+        node.tag_name().name()
+    }
+
+    let data = "<e xmlns='http://www.w3.org' />";
+    let doc = roxmltree::Document::parse(data).unwrap();
+    let root = doc.root_element();
+    assert_eq!(get_tag_name(&root), "e");
+}
