@@ -1,13 +1,12 @@
 extern crate roxmltree;
-#[macro_use] extern crate pretty_assertions;
 
 use roxmltree::*;
 
-use std::path;
+use std::fmt;
+use std::fmt::Write;
 use std::fs;
 use std::io::Read;
-use std::fmt::Write;
-use std::fmt;
+use std::path;
 
 #[derive(Clone, Copy, PartialEq)]
 struct TStr<'a>(pub &'a str);
@@ -18,45 +17,31 @@ impl<'a> fmt::Debug for TStr<'a> {
     }
 }
 
-
 trait HasExtension {
     fn has_extension(&self, ext: &str) -> bool;
 }
 
 impl HasExtension for path::Path {
     fn has_extension(&self, ext: &str) -> bool {
-        if let Some(e) = self.extension() { e == ext } else { false }
-    }
-}
-
-
-// List of not yet supported test cases.
-static IGNORE: &[&str] = &[
-    "tree_003.xml",
-];
-
-
-#[test]
-fn compare_ast() {
-    for entry in fs::read_dir("tests/files").unwrap() {
-        let entry = entry.unwrap();
-
-        if !entry.path().has_extension("xml") {
-            continue;
-        }
-
-        let file_name = entry.path().file_name().unwrap().to_str().unwrap().to_string();
-        if !IGNORE.contains(&file_name.as_str()) {
-            actual_test(entry.path());
+        if let Some(e) = self.extension() {
+            e == ext
+        } else {
+            false
         }
     }
 }
 
-fn actual_test(path: path::PathBuf) {
+fn actual_test(path: &str) {
+    let path = path::Path::new(path);
     let expected = load_file(&path.with_extension("yaml"));
 
+    let opt = ParsingOptions {
+        allow_dtd: true,
+        ..roxmltree::ParsingOptions::default()
+    };
+
     let input_xml = load_file(&path);
-    let doc = match Document::parse(&input_xml) {
+    let doc = match Document::parse_with_options(&input_xml, opt) {
         Ok(v) => v,
         Err(e) => {
             assert_eq!(TStr(&format!("error: \"{}\"", e)), TStr(expected.trim()));
@@ -105,20 +90,36 @@ fn _to_yaml(doc: &Document, s: &mut String) -> Result<(), fmt::Error> {
                     match child.tag_name().namespace() {
                         Some(ns) => {
                             if ns.is_empty() {
-                                writeln_indented!(depth + 2, s, "tag_name: {}", child.tag_name().name());
+                                writeln_indented!(
+                                    depth + 2,
+                                    s,
+                                    "tag_name: {}",
+                                    child.tag_name().name()
+                                );
                             } else {
-                                writeln_indented!(depth + 2, s, "tag_name: {}@{}",
-                                                  child.tag_name().name(), ns);
+                                writeln_indented!(
+                                    depth + 2,
+                                    s,
+                                    "tag_name: {}@{}",
+                                    child.tag_name().name(),
+                                    ns
+                                );
                             }
                         }
                         None => {
-                            writeln_indented!(depth + 2, s, "tag_name: {}", child.tag_name().name());
+                            writeln_indented!(
+                                depth + 2,
+                                s,
+                                "tag_name: {}",
+                                child.tag_name().name()
+                            );
                         }
                     }
 
-                    if !child.attributes().is_empty() {
+                    let attributes = child.attributes();
+                    if !(attributes.len() == 0) {
                         let mut attrs = Vec::new();
-                        for attr in child.attributes() {
+                        for attr in attributes {
                             match attr.namespace() {
                                 Some(ns) => {
                                     attrs.push((format!("{}@{}", attr.name(), ns), attr.value()));
@@ -140,7 +141,11 @@ fn _to_yaml(doc: &Document, s: &mut String) -> Result<(), fmt::Error> {
                         let mut ns_list = Vec::new();
                         for ns in child.namespaces() {
                             let name = ns.name().unwrap_or("None");
-                            let uri = if ns.uri().is_empty() { "\"\"" } else { ns.uri() };
+                            let uri = if ns.uri().is_empty() {
+                                "\"\""
+                            } else {
+                                ns.uri()
+                            };
                             ns_list.push((name, uri));
                         }
                         ns_list.sort_by(|a, b| a.0.cmp(&b.0));
@@ -187,3 +192,98 @@ fn _to_yaml(doc: &Document, s: &mut String) -> Result<(), fmt::Error> {
 
     Ok(())
 }
+
+macro_rules! test {
+    ($name:ident) => {
+        #[test]
+        fn $name() {
+            actual_test(&format!("tests/files/{}.xml", stringify!($name)))
+        }
+    };
+}
+
+test!(attrs_001);
+test!(attrs_002);
+test!(attrs_003);
+test!(attrs_004);
+test!(attrs_005);
+test!(attrs_006);
+test!(attrs_err_001);
+test!(attrs_err_002);
+test!(cdata_001);
+test!(cdata_002);
+test!(cdata_003);
+test!(cdata_004);
+test!(cdata_005);
+test!(cdata_006);
+test!(comments_001);
+test!(elems_err_001);
+test!(elems_err_002);
+test!(entity_001);
+test!(entity_002);
+test!(entity_003);
+test!(entity_004);
+test!(entity_005);
+test!(entity_006);
+test!(entity_007);
+test!(entity_008);
+test!(entity_009);
+test!(entity_010);
+test!(entity_011);
+test!(entity_012);
+test!(entity_013);
+test!(entity_014);
+test!(entity_err_001);
+test!(entity_err_002);
+test!(entity_err_003);
+test!(entity_err_004);
+test!(entity_err_005);
+test!(entity_err_006);
+test!(entity_err_007);
+test!(entity_err_008);
+test!(entity_err_009);
+test!(ns_001);
+test!(ns_002);
+test!(ns_003);
+test!(ns_004);
+test!(ns_005);
+test!(ns_006);
+test!(ns_007);
+test!(ns_008);
+test!(ns_009);
+test!(ns_010);
+test!(ns_011);
+test!(ns_012);
+test!(ns_013);
+test!(ns_014);
+test!(ns_015);
+test!(ns_016);
+test!(ns_017);
+test!(ns_err_001);
+test!(ns_err_002);
+test!(ns_err_003);
+test!(ns_err_004);
+test!(ns_err_005);
+test!(ns_err_006);
+test!(ns_err_007);
+test!(ns_err_008);
+test!(ns_err_009);
+test!(ns_err_010);
+test!(ns_err_011);
+test!(ns_err_012);
+test!(ns_err_013);
+test!(text_001);
+test!(text_002);
+test!(text_003);
+test!(text_004);
+test!(text_005);
+test!(text_006);
+test!(text_007);
+test!(text_008);
+test!(text_009);
+test!(text_010);
+test!(text_011);
+test!(tree_001);
+test!(tree_002);
+// test!(tree_003); // unsupported
+test!(tree_err_001);
